import os
import re
import secrets
import shutil

from InquirerPy import inquirer

from dj_beat_drop.utils import (
    green,
    red,
    get_latest_django_version,
    get_lts_django_version,
)


def get_secret_key():
    """
    Return a 50 character random string usable as a SECRET_KEY setting value.
    """
    chars = "abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)"
    return "".join(secrets.choice(chars) for _ in range(50))


def rename_template_files(project_dir):
    # Rename .py-tpl files to .py
    for root, _, files in os.walk(project_dir):
        for file in files:
            if file.endswith(".py-tpl"):
                old_file = os.path.join(root, file)
                new_file = os.path.join(root, file[:-4])
                os.rename(old_file, new_file)


def replace_settings_with_environs(content: str) -> str:
    rtn_val = content
    init_env = "# Initialize environs\n" "env = Env()\n" "env.read_env()"
    rtn_val = f"from environs import Env\n\n{rtn_val}"
    rtn_val = re.sub(r"(^BASE_DIR.+$)", rf"\1\n\n\n{init_env}", rtn_val, flags=re.MULTILINE)
    rtn_val = re.sub(
        r"^SECRET_KEY =.+$",
        'SECRET_KEY = env.str("SECRET_KEY")',
        rtn_val,
        flags=re.MULTILINE,
    )
    rtn_val = re.sub(r"^DEBUG =.+$", 'DEBUG = env.bool("DEBUG")', rtn_val, flags=re.MULTILINE)
    rtn_val = re.sub(
        r"^ALLOWED_HOSTS = .+$",
        'ALLOWED_HOSTS = env.list("ALLOWED_HOSTS")',
        rtn_val,
        flags=re.MULTILINE,
    )
    rtn_val = re.sub(
        r"^DATABASES\s*=\s*\{.+?\}\n\}",
        r'DATABASES = {"default": env.dj_db_url("DATABASE_URL")}',
        rtn_val,
        flags=re.MULTILINE | re.DOTALL,
    )
    return rtn_val


def create_dot_envfile(project_dir, context: dict[str, str]):
    env_file_path = os.path.join(project_dir, ".env")
    env_content = (
        "DEBUG=True\n"
        f"SECRET_KEY=\"{context['secret_key']}\"\n"
        f'ALLOWED_HOSTS=\n'
        f"DATABASE_URL=sqlite:///{os.path.join(project_dir, 'db.sqlite3')}"
    )
    with open(env_file_path, "w") as f:
        f.write(env_content)


def replace_variables(project_dir, context: dict[str, str], initialize_env):
    for root, _, files in os.walk(project_dir):
        for file in files:
            file_path = os.path.join(root, file)
            with open(file_path, "r") as f:
                content = f.read()
            for variable, value in context.items():
                content = content.replace(f"{{{{ {variable} }}}}", value)
            if file_path.endswith("config/settings.py") and initialize_env is True:
                content = replace_settings_with_environs(content)
                create_dot_envfile(project_dir, context)
            with open(file_path, "w") as f:
                f.write(content)


def handle_new(name, use_lts, overwrite_target_dir):
    if name is None:
        name = inquirer.text("Project name:").execute()

    if re.match(r"^[-a-z_]+$", name) is None:
        red("Invalid project name. Please use only lowercase letters, hyphens, and underscores.")
        return

    django_version, minor_version = get_latest_django_version()
    if use_lts is True:
        django_version, minor_version = get_lts_django_version()
    project_dir = os.path.join(os.getcwd(), name)
    template_dir_src = os.path.join(os.path.dirname(os.path.abspath(__file__)), "templates", minor_version)

    if os.path.exists(project_dir):
        if overwrite_target_dir is False:
            overwrite_response = inquirer.confirm(
                message=f"The directory '{name}' already exists. Do you want to overwrite it?",
                default=True,
            ).execute()
            if overwrite_response is False:
                red("Operation cancelled.")
                return
        shutil.rmtree(project_dir)

    initialize_uv = inquirer.confirm(message="Initialize your project with UV?", default=True).execute()
    initialize_env = inquirer.confirm(
        message="Initialize your project with an .env file and environs?", default=True
    ).execute()

    shutil.copytree(str(template_dir_src), project_dir)
    os.rename(os.path.join(project_dir, "project_name"), os.path.join(project_dir, "config"))

    rename_template_files(project_dir)
    replace_variables(
        project_dir,
        {
            "project_name": "config",
            "django_version": django_version,
            "docs_version": minor_version,
            "secret_key": get_secret_key(),
        },
        initialize_env,
    )

    if initialize_uv is True:
        os.chdir(project_dir)
        os.system("uv init")
        os.system("rm hello.py")
        os.system(f"uv add django~='{minor_version}'")
        if initialize_env is True:
            os.system("uv add environs[django]")
        os.system("uv run manage.py migrate")

    green("New Django project created.\n")

    if initialize_uv is True:
        green("To start Django's run server:\n")
        print(f"cd {name}")
        print("uv run manage.py runserver")



from pathlib import Path

from dj_beat_drop.new import replace_settings_with_environs
from dj_beat_drop.utils import get_latest_django_version


def test_replace_settings_with_environs():
    _, minor_version = get_latest_django_version()
    current_dir = Path(__file__).parent.parent
    settings_file_path = current_dir.joinpath(
        f"src/dj_beat_drop/templates/{minor_version}/project_name/settings.py-tpl"
    )
    print(settings_file_path)
    with open(settings_file_path, "r") as f:
        env_settings_content = f.read()
    result = replace_settings_with_environs(env_settings_content)

    assert "from environs import Env" in result
    assert "env = Env()" in result
    assert "env.read_env()" in result
    assert 'SECRET_KEY = env.str("SECRET_KEY")' in result
    assert 'DEBUG = env.bool("DEBUG")' in result
    assert 'ALLOWED_HOSTS = env.list("ALLOWED_HOSTS")' in result
    assert r'DATABASES = {"default": env.dj_db_url("DATABASE_URL")}' in result



from functools import lru_cache

import requests


def green(text):
    print(f"\033[92m{text}\033[0m")


def red(text):
    print(f"\033[91m{text}\033[0m")


@lru_cache
def get_django_releases():
    response = requests.get("https://pypi.org/pypi/Django/json")
    if response.status_code == 200:
        data = response.json()
        return {"latest": data["info"]["version"], "releases": data["releases"]}
    else:
        raise Exception("Failed to fetch Django releases")


def get_latest_django_version():
    full_version = get_django_releases()["latest"]
    minor_version = ".".join(full_version.split(".")[0:2])
    return full_version, minor_version


def get_lts_django_version():
    release_data = get_django_releases()
    latest_version = release_data["latest"]
    latest_minor_version = ".".join(latest_version.split(".")[0:2])
    latest_split = latest_version.split(".")
    if latest_split[1] == "2":
        return latest_version, latest_minor_version

    releases = get_django_releases()["releases"]
    for release in reversed(releases):
        if "b" in release or "rc" in release or "a" in release:
            continue
        version_parts = release.split(".")
        if version_parts[1] == "2":
            return release, ".".join(version_parts[0:2])



import argparse

import requests

from dj_beat_drop.new import handle_new
from pkg_resources import get_distribution, parse_version

from dj_beat_drop.utils import green


def get_ascii_logo():
    logo = r"""
   ___     __  ___           __    ___              
  / _ \__ / / / _ )___ ___ _/ /_  / _ \_______  ___ 
 / // / // / / _  / -_) _ `/ __/ / // / __/ _ \/ _ \
/____/\___/ /____/\__/\_,_/\__/ /____/_/  \___/ .__/
                                             /_/
"""
    return logo


def check_version():
    package_name = "dj-beat-drop"
    current_version = get_distribution(package_name).version

    response = requests.get(f"https://pypi.org/pypi/{package_name}/json")
    latest_version = response.json()["info"]["version"]

    if parse_version(current_version) < parse_version(latest_version):
        green(
            f"\033[0;33m\nA new version of {package_name} is available ({latest_version}). You are using {current_version}. To update, run:\n\033[0m"
        )
        print(f"  pip install --upgrade {package_name}\n")


def main():
    print(f"\033[38;2;255;165;0m{get_ascii_logo()}\033[0m")
    check_version()
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=f"Version {get_distribution('dj-beat-drop').version}",
    )
    subparsers = parser.add_subparsers(dest="command")

    # Add 'new' subcommand
    new_parser = subparsers.add_parser("new", help="Create a new Django project.")
    new_parser.add_argument(
        "name",
        type=str,
        nargs="?",
        help="Project name (e.g. 'example_project' or 'example-project').",
    )
    new_parser.add_argument(
        "--lts",
        action="store_true",
        help="Use the latest LTS version of Django.",
    )
    new_parser.add_argument(
        "--overwrite",
        action="store_true",
        help="Overwrite the project directory if it already exists.",
    )

    args = parser.parse_args()

    if args.command == "new":
        handle_new(args.name, args.lts, args.overwrite)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()
